/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSLevelIndicatorCell;

import dstep.appkit.NSActionCell;
import dstep.appkit.NSImage;
import dstep.appkit.NSSliderCell;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObjCRuntime;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias NSUInteger NSLevelIndicatorStyle;

enum
{
	NSRelevancyLevelIndicatorStyle,
	NSContinuousCapacityLevelIndicatorStyle,
	NSDiscreteCapacityLevelIndicatorStyle,
	NSRatingLevelIndicatorStyle
}

class NSLevelIndicatorCell : NSActionCell
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	NSLevelIndicatorCell initWithLevelIndicatorStyle (uint levelIndicatorStyle)
	{
		id result = invokeObjcSelf!(id, "initWithLevelIndicatorStyle:", uint)(levelIndicatorStyle);
		return result is this.objcObject ? this : (result !is null ? new NSLevelIndicatorCell(result) : null);
	}

	this (uint levelIndicatorStyle)
	{
		super(NSLevelIndicatorCell.alloc.initWithLevelIndicatorStyle(levelIndicatorStyle).objcObject);
	}

	uint levelIndicatorStyle ()
	{
		return invokeObjcSelf!(uint, "levelIndicatorStyle");
	}

	void setLevelIndicatorStyle (uint levelIndicatorStyle)
	{
		return invokeObjcSelf!(void, "setLevelIndicatorStyle:", uint)(levelIndicatorStyle);
	}

	double minValue ()
	{
		return invokeObjcSelf!(double, "minValue");
	}

	void setMinValue (double minValue)
	{
		return invokeObjcSelf!(void, "setMinValue:", double)(minValue);
	}

	double maxValue ()
	{
		return invokeObjcSelf!(double, "maxValue");
	}

	void setMaxValue (double maxValue)
	{
		return invokeObjcSelf!(void, "setMaxValue:", double)(maxValue);
	}

	double warningValue ()
	{
		return invokeObjcSelf!(double, "warningValue");
	}

	void setWarningValue (double warningValue)
	{
		return invokeObjcSelf!(void, "setWarningValue:", double)(warningValue);
	}

	double criticalValue ()
	{
		return invokeObjcSelf!(double, "criticalValue");
	}

	void setCriticalValue (double criticalValue)
	{
		return invokeObjcSelf!(void, "setCriticalValue:", double)(criticalValue);
	}

	void setTickMarkPosition (uint position)
	{
		return invokeObjcSelf!(void, "setTickMarkPosition:", uint)(position);
	}

	uint tickMarkPosition ()
	{
		return invokeObjcSelf!(uint, "tickMarkPosition");
	}

	void setNumberOfTickMarks (NSInteger count)
	{
		return invokeObjcSelf!(void, "setNumberOfTickMarks:", NSInteger)(count);
	}

	NSInteger numberOfTickMarks ()
	{
		return invokeObjcSelf!(NSInteger, "numberOfTickMarks");
	}

	void setNumberOfMajorTickMarks (NSInteger count)
	{
		return invokeObjcSelf!(void, "setNumberOfMajorTickMarks:", NSInteger)(count);
	}

	NSInteger numberOfMajorTickMarks ()
	{
		return invokeObjcSelf!(NSInteger, "numberOfMajorTickMarks");
	}

	NSRect rectOfTickMarkAtIndex (NSInteger index)
	{
		return invokeObjcSelf!(NSRect, "rectOfTickMarkAtIndex:", NSInteger)(index);
	}

	double tickMarkValueAtIndex (NSInteger index)
	{
		return invokeObjcSelf!(double, "tickMarkValueAtIndex:", NSInteger)(index);
	}

	void setImage (NSImage image)
	{
		return invokeObjcSelf!(void, "setImage:", NSImage)(image);
	}
}

