/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSSegmentedControl;

import dstep.appkit.NSCell;
import dstep.appkit.NSControl;
import dstep.appkit.NSImage;
import dstep.appkit.NSMenu;
import dstep.applicationservices.coregraphics.CGBase;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

typedef NSInteger NSSegmentStyle;

enum : NSInteger
{
	NSSegmentStyleAutomatic = 0,
	NSSegmentStyleRounded = 1,
	NSSegmentStyleTexturedRounded = 2,
	NSSegmentStyleRoundRect = 3,
	NSSegmentStyleTexturedSquare = 4,
	NSSegmentStyleCapsule = 5,
	NSSegmentStyleSmallSquare = 6
}

class NSSegmentedControl : NSControl
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	void setSegmentCount (NSInteger count)
	{
		return invokeObjcSelf!(void, "setSegmentCount:", NSInteger)(count);
	}

	NSInteger segmentCount ()
	{
		return invokeObjcSelf!(NSInteger, "segmentCount");
	}

	void setSelectedSegment (NSInteger selectedSegment)
	{
		return invokeObjcSelf!(void, "setSelectedSegment:", NSInteger)(selectedSegment);
	}

	NSInteger selectedSegment ()
	{
		return invokeObjcSelf!(NSInteger, "selectedSegment");
	}

	bool selectSegmentWithTag (NSInteger tag)
	{
		return invokeObjcSelf!(bool, "selectSegmentWithTag:", NSInteger)(tag);
	}

	void setWidth (CGFloat width, NSInteger segment)
	{
		return invokeObjcSelf!(void, "setWidth:forSegment:", CGFloat, NSInteger)(width, segment);
	}

	CGFloat widthForSegment (NSInteger segment)
	{
		return invokeObjcSelf!(CGFloat, "widthForSegment:", NSInteger)(segment);
	}

	void setImage (NSImage image, NSInteger segment)
	{
		return invokeObjcSelf!(void, "setImage:forSegment:", NSImage, NSInteger)(image, segment);
	}

	NSImage imageForSegment (NSInteger segment)
	{
		return invokeObjcSelf!(NSImage, "imageForSegment:", NSInteger)(segment);
	}

	void setImageScaling (uint scaling, NSInteger segment)
	{
		return invokeObjcSelf!(void, "setImageScaling:forSegment:", uint, NSInteger)(scaling, segment);
	}

	uint imageScalingForSegment (NSInteger segment)
	{
		return invokeObjcSelf!(uint, "imageScalingForSegment:", NSInteger)(segment);
	}

	void setLabel (NSString label, NSInteger segment)
	{
		return invokeObjcSelf!(void, "setLabel:forSegment:", NSString, NSInteger)(label, segment);
	}

	NSString labelForSegment (NSInteger segment)
	{
		return invokeObjcSelf!(NSString, "labelForSegment:", NSInteger)(segment);
	}

	void setMenu (NSMenu menu, NSInteger segment)
	{
		return invokeObjcSelf!(void, "setMenu:forSegment:", NSMenu, NSInteger)(menu, segment);
	}

	NSMenu menuForSegment (NSInteger segment)
	{
		return invokeObjcSelf!(NSMenu, "menuForSegment:", NSInteger)(segment);
	}

	void setSelected (bool selected, NSInteger segment)
	{
		return invokeObjcSelf!(void, "setSelected:forSegment:", bool, NSInteger)(selected, segment);
	}

	bool isSelectedForSegment (NSInteger segment)
	{
		return invokeObjcSelf!(bool, "isSelectedForSegment:", NSInteger)(segment);
	}

	void setEnabled (bool enabled, NSInteger segment)
	{
		return invokeObjcSelf!(void, "setEnabled:forSegment:", bool, NSInteger)(enabled, segment);
	}

	bool isEnabledForSegment (NSInteger segment)
	{
		return invokeObjcSelf!(bool, "isEnabledForSegment:", NSInteger)(segment);
	}

	void setSegmentStyle (int segmentStyle)
	{
		return invokeObjcSelf!(void, "setSegmentStyle:", int)(segmentStyle);
	}

	int segmentStyle ()
	{
		return invokeObjcSelf!(int, "segmentStyle");
	}
}

