/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSUserDefaultsController;

import dstep.appkit.NSController;
import dstep.foundation.NSUserDefaults;
import dstep.foundation.NSDictionary;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSUserDefaultsController : NSController
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	static Object sharedUserDefaultsController ()
	{
		return invokeObjcSelfClass!(Object, "sharedUserDefaultsController");
	}

	NSUserDefaultsController initWithDefaults (NSUserDefaults defaults, NSDictionary initialValues)
	{
		id result = invokeObjcSelf!(id, "initWithDefaults:initialValues:", NSUserDefaults, NSDictionary)(defaults, initialValues);
		return result is this.objcObject ? this : (result !is null ? new NSUserDefaultsController(result) : null);
	}

	this (NSUserDefaults defaults, NSDictionary initialValues)
	{
		super(NSUserDefaultsController.alloc.initWithDefaults(defaults, initialValues).objcObject);
	}

	NSUserDefaults defaults ()
	{
		return invokeObjcSelf!(NSUserDefaults, "defaults");
	}

	void setInitialValues (NSDictionary initialValues)
	{
		return invokeObjcSelf!(void, "setInitialValues:", NSDictionary)(initialValues);
	}

	NSUserDefaultsController initialValues ()
	{
		id result = invokeObjcSelf!(id, "initialValues");
		return result is this.objcObject ? this : (result !is null ? new NSUserDefaultsController(result) : null);
	}

	this ()
	{
		super(NSUserDefaultsController.alloc.initialValues.objcObject);
	}

	void setAppliesImmediately (bool flag)
	{
		return invokeObjcSelf!(void, "setAppliesImmediately:", bool)(flag);
	}

	bool appliesImmediately ()
	{
		return invokeObjcSelf!(bool, "appliesImmediately");
	}

	bool hasUnappliedChanges ()
	{
		return invokeObjcSelf!(bool, "hasUnappliedChanges");
	}

	Object values ()
	{
		return invokeObjcSelf!(Object, "values");
	}

	void revert (Object sender)
	{
		return invokeObjcSelf!(void, "revert:", Object)(sender);
	}

	void save (Object sender)
	{
		return invokeObjcSelf!(void, "save:", Object)(sender);
	}

	void revertToInitialValues (Object sender)
	{
		return invokeObjcSelf!(void, "revertToInitialValues:", Object)(sender);
	}
}

