/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 25, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.coredata.NSEntityDescription;

import dstep.coredata.NSManagedObject;
import dstep.coredata.NSManagedObjectContext;
import dstep.coredata.NSManagedObjectModel;
import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSData;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSEnumerator;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSRange;
import dstep.foundation.NSString;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSEntityDescription : NSObject, INSCoding, INSCopying, INSFastEnumeration
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	NSEntityDescription initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(NSEntityDescription, "initWithCoder:", NSCoder)(aDecoder);
	}
	
	NSObject copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(NSObject, "copyWithZone:", NSZone*)(zone);
	}
	
	NSUInteger countByEnumeratingWithState (NSFastEnumerationState* state, id* stackbuf, NSUInteger len)
	{
		return invokeObjcSelf!(NSUInteger, "countByEnumeratingWithState:objects:count:", NSFastEnumerationState*, id*, NSUInteger)(state, stackbuf, len);
	}

	static NSEntityDescription entityForName (NSString entityName, NSManagedObjectContext context)
	{
		return invokeObjcSelfClass!(NSEntityDescription, "entityForName:inManagedObjectContext:", NSString, NSManagedObjectContext)(entityName, context);
	}

	static Object insertNewObjectForEntityForName (NSString entityName, NSManagedObjectContext context)
	{
		return invokeObjcSelfClass!(Object, "insertNewObjectForEntityForName:inManagedObjectContext:", NSString, NSManagedObjectContext)(entityName, context);
	}

	NSManagedObjectModel managedObjectModel ()
	{
		return invokeObjcSelf!(NSManagedObjectModel, "managedObjectModel");
	}

	NSString managedObjectClassName ()
	{
		return invokeObjcSelf!(NSString, "managedObjectClassName");
	}

	void setManagedObjectClassName (NSString name)
	{
		return invokeObjcSelf!(void, "setManagedObjectClassName:", NSString)(name);
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	void setName (NSString name)
	{
		return invokeObjcSelf!(void, "setName:", NSString)(name);
	}

	bool isAbstract ()
	{
		return invokeObjcSelf!(bool, "isAbstract");
	}

	void setAbstract (bool flag)
	{
		return invokeObjcSelf!(void, "setAbstract:", bool)(flag);
	}

	NSDictionary subentitiesByName ()
	{
		return invokeObjcSelf!(NSDictionary, "subentitiesByName");
	}

	NSArray subentities ()
	{
		return invokeObjcSelf!(NSArray, "subentities");
	}

	void setSubentities (NSArray array)
	{
		return invokeObjcSelf!(void, "setSubentities:", NSArray)(array);
	}

	NSEntityDescription superentity ()
	{
		id result = invokeObjcSelf!(id, "superentity");
		return result is this.objcObject ? this : (result !is null ? new NSEntityDescription(result) : null);
	}

	NSDictionary propertiesByName ()
	{
		return invokeObjcSelf!(NSDictionary, "propertiesByName");
	}

	NSArray properties ()
	{
		return invokeObjcSelf!(NSArray, "properties");
	}

	void setProperties (NSArray properties)
	{
		return invokeObjcSelf!(void, "setProperties:", NSArray)(properties);
	}

	NSDictionary userInfo ()
	{
		return invokeObjcSelf!(NSDictionary, "userInfo");
	}

	void setUserInfo (NSDictionary dictionary)
	{
		return invokeObjcSelf!(void, "setUserInfo:", NSDictionary)(dictionary);
	}

	NSDictionary attributesByName ()
	{
		return invokeObjcSelf!(NSDictionary, "attributesByName");
	}

	NSDictionary relationshipsByName ()
	{
		return invokeObjcSelf!(NSDictionary, "relationshipsByName");
	}

	NSArray relationshipsWithDestinationEntity (NSEntityDescription entity)
	{
		return invokeObjcSelf!(NSArray, "relationshipsWithDestinationEntity:", NSEntityDescription)(entity);
	}

	bool isKindOfEntity (NSEntityDescription entity)
	{
		return invokeObjcSelf!(bool, "isKindOfEntity:", NSEntityDescription)(entity);
	}

	NSData versionHash ()
	{
		return invokeObjcSelf!(NSData, "versionHash");
	}

	NSString versionHashModifier ()
	{
		return invokeObjcSelf!(NSString, "versionHashModifier");
	}

	void setVersionHashModifier (NSString modifierString)
	{
		return invokeObjcSelf!(void, "setVersionHashModifier:", NSString)(modifierString);
	}

}

