/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSClassDescription;

import dstep.foundation.NSArray;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSException;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;



import bindings = dstep.foundation.NSClassDescription_bindings;

private NSString NSClassDescriptionNeededForClassNotification_;

NSString NSClassDescriptionNeededForClassNotification ()
{
	if (NSClassDescriptionNeededForClassNotification_)
		return NSClassDescriptionNeededForClassNotification_;
	
	return NSClassDescriptionNeededForClassNotification_ = new NSString(bindings.NSClassDescriptionNeededForClassNotification);
}

const TNSClassDescriptionPrimitives = `

	NSClassDescription classDescription ()
	{
		return invokeObjcSelf!(NSClassDescription, "classDescription");
	}
	
	NSArray attributeKeys ()
	{
		return invokeObjcSelf!(NSArray, "attributeKeys");
	}
	
	NSArray toOneRelationshipKeys ()
	{
		return invokeObjcSelf!(NSArray, "toOneRelationshipKeys");
	}
	
	NSArray toManyRelationshipKeys ()
	{
		return invokeObjcSelf!(NSArray, "toManyRelationshipKeys");
	}
	
	NSString inverseForRelationshipKey (NSString relationshipKey)
	{
		return invokeObjcSelf!(NSString, "inverseForRelationshipKey:", NSString)(relationshipKey);
	}
	
	//mixin ObjcBindMethod!(classDescription, "classDescription");
	//mixin ObjcBindMethod!(attributeKeys, "attributeKeys");
	//mixin ObjcBindMethod!(toOneRelationshipKeys, "toOneRelationshipKeys");
	//mixin ObjcBindMethod!(toManyRelationshipKeys, "toManyRelationshipKeys");
	//mixin ObjcBindMethod!(inverseForRelationshipKey, "inverseForRelationshipKey:");
`;

class NSClassDescription : NSObject
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static void registerClassDescription (NSClassDescription description, Class aClass)
	{
		return invokeObjcSuperClass!(void, "registerClassDescription:forClass:", NSClassDescription, Class)(description, aClass);
	}

	static void invalidateClassDescriptionCache ()
	{
		return invokeObjcSuperClass!(void, "invalidateClassDescriptionCache");
	}

	static NSClassDescription classDescriptionForClass (Class aClass)
	{
		return invokeObjcSuperClass!(NSClassDescription, "classDescriptionForClass:", Class)(aClass);
	}

	NSArray attributeKeys ()
	{
		return invokeObjcSelf!(NSArray, "attributeKeys");
	}

	NSArray toOneRelationshipKeys ()
	{
		return invokeObjcSelf!(NSArray, "toOneRelationshipKeys");
	}

	NSArray toManyRelationshipKeys ()
	{
		return invokeObjcSelf!(NSArray, "toManyRelationshipKeys");
	}

	NSString inverseForRelationshipKey (NSString relationshipKey)
	{
		return invokeObjcSelf!(NSString, "inverseForRelationshipKey:", NSString)(relationshipKey);
	}
}