/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSIndexPath;

import dstep.foundation.NSCoder;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSIndexPath : NSObject, INSCopying, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}
	
	static Object indexPathWithIndex (NSUInteger index)
	{
		return invokeObjcSuperClass!(Object, "indexPathWithIndex:", NSUInteger)(index);
	}

	static Object indexPathWithIndexes (NSUInteger* indexes, NSUInteger length)
	{
		return invokeObjcSuperClass!(Object, "indexPathWithIndexes:length:", NSUInteger*, NSUInteger)(indexes, length);
	}

	Object initWithIndex (NSUInteger index)
	{
		return invokeObjcSelf!(Object, "initWithIndex:", NSUInteger)(index);
	}

	this (NSUInteger index)
	{
		typeof(this).alloc.initWithIndex(index);
	}

	Object initWithIndexes (NSUInteger* indexes, NSUInteger length)
	{
		return invokeObjcSelf!(Object, "initWithIndexes:length:", NSUInteger*, NSUInteger)(indexes, length);
	}

	this (NSUInteger* indexes, NSUInteger length)
	{
		typeof(this).alloc.initWithIndexes(indexes, length);
	}

	NSIndexPath indexPathByAddingIndex (NSUInteger index)
	{
		id result = invokeObjcSelf!(id, "indexPathByAddingIndex:", NSUInteger)(index);
		return result is this.objcObject ? this : (result !is null ? new NSIndexPath(result) : null);
	}

	NSIndexPath indexPathByRemovingLastIndex ()
	{
		id result = invokeObjcSelf!(id, "indexPathByRemovingLastIndex");
		return result is this.objcObject ? this : (result !is null ? new NSIndexPath(result) : null);
	}

	NSUInteger indexAtPosition (NSUInteger position)
	{
		return invokeObjcSelf!(NSUInteger, "indexAtPosition:", NSUInteger)(position);
	}

	NSUInteger length ()
	{
		return invokeObjcSelf!(NSUInteger, "length");
	}

	void getIndexes (NSUInteger* indexes)
	{
		return invokeObjcSelf!(void, "getIndexes:", NSUInteger*)(indexes);
	}

	int compare (NSIndexPath otherObject)
	{
		return invokeObjcSelf!(int, "compare:", NSIndexPath)(otherObject);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		typeof(this).alloc.initWithCoder(aDecoder);
	}
}

