/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.quartzcore.CIFilterShape;

import dstep.applicationservices.ApplicationServices;
import dstep.foundation.NSObject;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class CIFilterShape : NSObject, INSCopying
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	typeof(this) copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(typeof(this), "copyWithZone:", NSZone*)(zone);
	}

	static Object shapeWithRect (CGRect r)
	{
		return invokeObjcSelfClass!(Object, "shapeWithRect:", CGRect)(r);
	}

	CIFilterShape initWithRect (CGRect r)
	{
		id result = invokeObjcSelf!(id, "initWithRect:", CGRect)(r);
		return result is this.objcObject ? this : (result !is null ? new CIFilterShape(result) : null);
	}

	this (CGRect r)
	{
		super(CIFilterShape.alloc.initWithRect(r).objcObject);
	}

	CIFilterShape transformBy (CGAffineTransform m, bool flag)
	{
		id result = invokeObjcSelf!(id, "transformBy:interior:", CGAffineTransform, bool)(m, flag);
		return result is this.objcObject ? this : (result !is null ? new CIFilterShape(result) : null);
	}

	CIFilterShape insetByX (int dx, int dy)
	{
		id result = invokeObjcSelf!(id, "insetByX:Y:", int, int)(dx, dy);
		return result is this.objcObject ? this : (result !is null ? new CIFilterShape(result) : null);
	}

	CIFilterShape unionWith (CIFilterShape s2)
	{
		id result = invokeObjcSelf!(id, "unionWith:", CIFilterShape)(s2);
		return result is this.objcObject ? this : (result !is null ? new CIFilterShape(result) : null);
	}

	CIFilterShape unionWithRect (CGRect r)
	{
		id result = invokeObjcSelf!(id, "unionWithRect:", CGRect)(r);
		return result is this.objcObject ? this : (result !is null ? new CIFilterShape(result) : null);
	}

	CIFilterShape intersectWith (CIFilterShape s2)
	{
		id result = invokeObjcSelf!(id, "intersectWith:", CIFilterShape)(s2);
		return result is this.objcObject ? this : (result !is null ? new CIFilterShape(result) : null);
	}

	CIFilterShape intersectWithRect (CGRect r)
	{
		id result = invokeObjcSelf!(id, "intersectWithRect:", CGRect)(r);
		return result is this.objcObject ? this : (result !is null ? new CIFilterShape(result) : null);
	}

}

